@@ -256,6 +256,7 @@ const AVCodecTag codec_movaudio_tags[] = {
     { CODEC_ID_PCM_MULAW,       MKTAG('u', 'l', 'a', 'w') },
     { CODEC_ID_PCM_S16BE,       MKTAG('t', 'w', 'o', 's') },
     { CODEC_ID_PCM_S16LE,       MKTAG('s', 'o', 'w', 't') },
+    { CODEC_ID_PCM_S16BE,       MKTAG('l', 'p', 'c', 'm') },
     { CODEC_ID_PCM_S16LE,       MKTAG('l', 'p', 'c', 'm') },
     { CODEC_ID_PCM_S24BE,       MKTAG('i', 'n', '2', '4') },
     { CODEC_ID_PCM_S24LE,       MKTAG('i', 'n', '2', '4') },
@@ -339,7 +340,7 @@ int ff_mov_lang_to_iso639(unsigned code, char to[4])
     memset(to, 0, 4);
     /* is it the mangled iso code? */
     /* see http://www.geocities.com/xhelmboyx/quicktime/formats/mp4-layout.txt */
-    if (code > 138) {
+    if (code >= 0x400 && code != 0x7fff) {
         for (i = 2; i >= 0; i--) {
             to[i] = 0x60 + (code & 0x1f);
             code >>= 5;
